from zope.i18n import translate
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid.discount')
from zope.component import getUtility

from zope import component
from getpaid.core import interfaces

from plone.app.layout.viewlets.common import ViewletBase

from getpaid.core.interfaces import IShoppingCartUtility

from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from getpaid.discount.browser.interfaces import IDiscountableMarker
from getpaid.discount.browser.interfaces import IDiscountable
from getpaid.discount.browser.interfaces import IBuyXGetXFreeable
from getpaid.discount.browser.interfaces import IBuyXGetXFreeableMarker
from getpaid.discount.browser.interfaces import ICodeDiscountable
from getpaid.discount.browser.interfaces import ICodeDiscountableMarker

from zope.annotation.interfaces import IAnnotations

class DiscountListingViewlet(ViewletBase):
    render = ViewPageTemplateFile('discount_listing.pt')
    
    def update(self):
        #self.portal_state = getMultiAdapter((self.context, self.request),
        #                                    name=u'plone_portal_state')
        #self.portal_url = self.portal_state.portal_url()
        self.cart = getUtility(IShoppingCartUtility).get(self.context) or {}
        # if cart is destroyed, we'll use the order_id to retrieve the cart detail and discount
        if not self.cart:
            order_id = self.request.get("order_id", None)
            if order_id:
                order_manager = component.getUtility(interfaces.IOrderManager)
                #self.order = order_manager.get(order_id)
                self.cart = order_manager.get(order_id).shopping_cart
        
    def getDiscounts(self):
        """
        for all the items in the cart
        we look if they have a discount
        and if so, we bring back a list of dictionaries
        """
        results = []
        if self.cart:
            for payable_line in self.cart.values():
                annotation = IAnnotations(payable_line)
                ref_obj = payable_line.resolve()
                payable_quantity = payable_line.quantity
                if ref_obj and IDiscountableMarker.providedBy(ref_obj):
                    adapter_obj = IDiscountable(ref_obj)
                    discount_value = adapter_obj.getDiscountValue()
                    if discount_value != 0.0:
                        discount_title = adapter_obj.getDiscountTitle()
                        discount_type = adapter_obj.getDiscountType()
                        if discount_type != 'Dollars Off':

                            msgid = _(u"total_discount_off", default= u"${discount_value_X}", mapping={ u"discount_value_X" : discount_value* payable_quantity})
                            description = translate(msgid, domain='getpaid.discount',context=self.request)
                            # total value of the discount would be a better word
                                                                                
                        else:
                            msgid = _(u"total_discount_percentage", default= u"Total of ${discount_value_percentage}", mapping={ u"discount_value_percentage" : discount_value* payable_quantity})
                            description = translate(msgid, domain='getpaid.discount',context=self.request)
                        msgid = _(u"discount_title_ref_obj_title", default= u"${discount_title} on ${ref_obj_title}", mapping={ u"discount_title" : discount_title,u"ref_obj_title":ref_obj.Title()})
                        title = translate(msgid, domain='getpaid.discount',context=self.request)  
                        res = {'title': title, 'description': description}
                        results.append(res)
                elif IBuyXGetXFreeableMarker.providedBy(ref_obj):
                    adapter_obj = IBuyXGetXFreeable(ref_obj)
                    number_to_buy = adapter_obj.getNumberToBuy()
                    number_free = adapter_obj.getNumberFree()
                    if number_to_buy != 0 and number_free != 0:
                        discount_title = adapter_obj.getDiscountTitle()
                        number_res = int(payable_quantity / number_to_buy * number_free)
                        msgid = _(u"free_additional_items_number", default=u"${number_res} free additional item(s)", mapping={ u"number_res":str(number_res)})
                        description = translate(msgid, domain='getpaid.discount',context=self.request)
                        msgid = _(u"free_additional_items_title", default= u"${discount_title} on ${ref_obj_title}", mapping={u"discount_title":discount_title,u"ref_obj_title":ref_obj.Title()})
  
                        res = {'title': translate(msgid, domain='getpaid.discount',context=self.request), 
                           'description': description
                          }
                    results.append(res)
                elif ICodeDiscountableMarker.providedBy(ref_obj) \
                     and "getpaid.discount.code" in annotation:

                    discount_title = annotation["getpaid.discount.code.title"]
                    discount_value = annotation["getpaid.discount.code.discount"]
                    if discount_value != 0.0:
                        msgid = _(u"total_percentage_off", default= u"Total of ${discount_value} off", mapping={ u"discount_value" : discount_value})
                        description = translate(msgid, domain='getpaid.discount', context=self.request)
                        msgid = _(u"total_percentage_off_title", default= u"${discount_title} on ${ref_obj_title} off", mapping={ u"discount_title" : discount_title,u"ref_obj_title":ref_obj.Title()})
                        title = translate(msgid, domain='getpaid.discount',context=self.request)
                        res = {'title': title, 
                               'description': description
                              }
                        results.append(res)

        return results

class DiscountCodeViewlet(ViewletBase):
    render = ViewPageTemplateFile('discount_code.pt')
    
    def update(self):
        #self.portal_state = getMultiAdapter((self.context, self.request),
        #                                    name=u'plone_portal_state')
        #self.portal_url = self.portal_state.portal_url()
        self.cart = getUtility(IShoppingCartUtility).get(self.context) or {}
        # if cart is destroyed, we'll use the order_id to retrieve the cart detail and discount
        if not self.cart:
            order_id = self.request.get("order_id", None)
            if order_id:
                order_manager = component.getUtility(interfaces.IOrderManager)
                #self.order = order_manager.get(order_id)
                self.cart = order_manager.get(order_id).shopping_cart
        




from zope import schema
from zope.interface import Interface
from getpaid.discount import discountMessageFactory as _
class IDiscountable(Interface):
    """
    Marker interface for the Discountable items
    """
    discount_title = schema.TextLine(title=_(u'Discount Title'),
                           required=True)
    
    discount_type = schema.Choice(title=_(u'Type of Discount'),
                                  values=[_(u'Dollars Off'), _(u'Percentage Off')],
                                  required=True)
    
    discount_value = schema.Float(title=_(u'Value of the discount'),
                                  required=True,)

class IDiscountableMarker(Interface):
    """
    Discount Interface
    """

class IBuyXGetXFreeable(Interface):
    """
    Marker interface for the BuyXGetXFreeable items
    """
    discount_title = schema.TextLine(title=_(u'Discount Title'),
                           required=True)
    
    number_to_buy = schema.Int(title=_(u'Number of items to buy'),
                                  required=True)
    
    number_free = schema.Int(title=_(u'Number of free items'),
                                  required=True)

class IBuyXGetXFreeableMarker(Interface):
    """
    Discount Interface
    """

class ICodeDiscountable(Interface):
    """
    Marker interface for the CodeDiscountable items
    """
    discount_title_1 = schema.TextLine(title=_(u'Discount Title'),
                           required=True)

    discount_code_1 = schema.TextLine(title=_(u'Discount Code'),
                           required=True)
    
    discounted_price_1 = schema.Float(title=_(u'Price after Discount'),
                                  required=True,)


    discount_title_2 = schema.TextLine(title=_(u'Second Discount Title'),
                           required=False)

    discount_code_2 = schema.TextLine(title=_(u'Second Discount Code'),
                           required=False)
    
    discounted_price_2 = schema.Float(title=_(u'Price after Second Discount'),
                                  required=False,)


    discount_title_3 = schema.TextLine(title=_(u'Third Discount Title'),
                           required=False)

    discount_code_3 = schema.TextLine(title=_(u'Third Discount Code'),
                           required=False)
    
    discounted_price_3 = schema.Float(title=_(u'Price after Third Discount'),
                                  required=False,)


class ICodeDiscountableMarker(Interface):
    """
    Discount Interface
    """



from StringIO import StringIO

try:
   from plone.app.upgrade import v40
   HAS_PLONE30 = True
   HAS_PLONE40 = True
except ImportError:
   HAS_PLONE40 = False
   try:
       from Products.CMFPlone.migrations import v3_0
   except ImportError:
       HAS_PLONE30 = False
   else:
       HAS_PLONE30 = True

def install_plone3_portlets(self):
    """Adds the discountable portlet to the root of the site
    so that it shows up on discountable items
    """
    if not HAS_PLONE30:
        return

    # Do the imports here, as we only need them here and this only
    # gets run on Plone 3.0.
    from zope.app.container.interfaces import INameChooser
    from zope.component import getUtility, getMultiAdapter
    from plone.portlets.interfaces import IPortletManager, IPortletAssignmentMapping
    from getpaid.discount.browser.portlets import portlets30

    # Get some definitions.
    portal = self.portal_url.getPortalObject()
    column = getUtility(IPortletManager, name="plone.rightcolumn", context=portal)
    manager = getMultiAdapter((portal, column), IPortletAssignmentMapping)
    portletnames = [v.title for v in manager.values()]
    chooser = INameChooser(manager)

    assignments = [
        portlets30.DiscountableAssignment(),
        portlets30.BuyXGetXfreeableAssignment(),
        ]

    for assignment in assignments:
        title = assignment.title
        if title not in portletnames:
            manager[chooser.chooseName(title, assignment)] = assignment

def setupVarious(context):
    """Import steps that are not handled by GS import/export handlers can be
    defined in the setupVarious() function.
    """
    if context.readDataFile('getpaid.discount-default.txt') is None:
        return

    site = context.getSite()

    logger = context.getLogger("getpaid.discount-default")
    out = StringIO()

    if HAS_PLONE30:
        print >> out, "Installing Discount Plone 3 Portlets"
        install_plone3_portlets(site)
    
    logger.info(out.getvalue())
    
    return "Setup various finished"




from Products.Five.browser import BrowserView

from getpaid.discount.browser.interfaces import IDiscountableMarker
from getpaid.discount.browser.interfaces import IBuyXGetXFreeableMarker
from getpaid.discount.browser.interfaces import ICodeDiscountableMarker

from Products.PloneGetPaid.interfaces import IPayableMarker

class ContentControl(BrowserView):
    """ conditions for presenting various actions
    """
    __allow_access_to_unprotected_subobjects__ = 1
    
    def __init__( self, context, request ):
        self.context = context
        self.request = request

    def isPossibleDiscountable(self):
        """  does the context implement the IPayableMarker interface
        """
        return IPayableMarker.providedBy(self.context) and \
            not IDiscountableMarker.providedBy(self.context) and \
            not IBuyXGetXFreeableMarker.providedBy(self.context) and \
            not ICodeDiscountableMarker.providedBy(self.context)
    
    isPossibleDiscountable.__roles__ = None
    
    def isDiscountable( self ):
        """  does the context implement the IDiscountableMarker interface
        """
        return IDiscountableMarker.providedBy(self.context)
    
    isDiscountable.__roles__ = None
    
    def isBuyXGetXfreeable(self):
        """does the context implement the IBuyXGetXFreeableMarker interface
        """
        return IBuyXGetXFreeableMarker.providedBy(self.context)

    isBuyXGetXfreeable.__roles__ = None

    def isCodeDiscountable(self):
        """does the context implement the ICodeDiscountableMarker interface
        """
        return ICodeDiscountableMarker.providedBy(self.context)

    isCodeDiscountable.__roles__ = None



from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid.discount')
from zope.i18n import translate
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile

from getpaid.discount.browser.interfaces import IDiscountableMarker
from getpaid.discount.browser.interfaces import IBuyXGetXFreeableMarker
from getpaid.discount.browser.portlets25 import BaseDiscountPortlet

from zope.interface import implements
from plone.portlets.interfaces import IPortletDataProvider
from plone.app.portlets.portlets import base

from Products.PloneGetPaid.interfaces import PayableMarkerMap
from Products.PloneGetPaid.interfaces import IPayableMarker
from Products.PloneGetPaid.browser.portlets import buy
from Products.PloneGetPaid.browser.portlets import donate
from Products.PloneGetPaid.browser.portlets import premium
from Products.PloneGetPaid.browser.portlets import ship

class DiscountRenderer(base.Renderer, BaseDiscountPortlet):
    """Plone 3.0 Base rendered useful for discount portlets.
    """

    # Marker interface that this renderer is meant for.
    marker = IPayableMarker
    # Supply a template id in the inheriting class.

    @property
    def available(self):
        """Portlet is available when a marker interface is provided.

        Overwrite this by picking a different interface.
        """
        return self.marker.providedBy(self.context)
        
    @property
    def payable(self):
        """Return the payable (shippable) version of the context.
        """
        iface = PayableMarkerMap.get(self.marker, None)
        if iface is None:
            # Something is badly wrong here.
            return None
        return iface( self.context )


class IDiscountablePortlet(IPortletDataProvider):
    pass


class DiscountableAssignment(base.Assignment):
    implements(IDiscountablePortlet)

    @property
    def title(self):
        """Title shown in @@manage-portlets.
        """
        msgid = _(u"Discountable_portlet", default= u"Discountable Portlet")
        result = translate(msgid, domain='getpaid.discount')

        return result


class DiscountableAddForm(base.NullAddForm):

    def create(self):
        return DiscountableAssignment()


class DiscountableRenderer(DiscountRenderer):
    marker = IDiscountableMarker
    render = ViewPageTemplateFile('../templates/portlet_content_discountable.pt')


class IBuyXGetXfreeablePortlet(IPortletDataProvider):
    pass


class BuyXGetXfreeableAssignment(base.Assignment):
    implements(IBuyXGetXfreeablePortlet)

    @property
    def title(self):
        """Title shown in @@manage-portlets.
        """
        return _(u"BuyXGetXFreeable")


class BuyXGetXfreeableAddForm(base.NullAddForm):

    def create(self):
        return BuyXGetXfreeableAssignment()


class BuyXGetXfreeableRenderer(DiscountRenderer):
    marker = IBuyXGetXFreeableMarker
    render = ViewPageTemplateFile('../templates/portlet_content_buyxgetxfreeable.pt')


#Plone 3 renderers for the portlets
class BuyRenderer(buy.Renderer):
    """"""
    
    @property
    def available(self):
        """"""
        return self.marker.providedBy(self.context) and \
            not IDiscountableMarker.providedBy(self.context) and \
            not IBuyXGetXFreeableMarker.providedBy(self.context)

class DonateRenderer(donate.Renderer):
    """"""
    
    @property
    def available(self):
        """"""
        return self.marker.providedBy(self.context) and \
            not IDiscountableMarker.providedBy(self.context) and \
            not IBuyXGetXFreeableMarker.providedBy(self.context)

class PremiumRenderer(premium.Renderer):
    """"""
    
    @property
    def available(self):
        """"""
        return self.marker.providedBy(self.context) and \
            not IDiscountableMarker.providedBy(self.context) and \
            not IBuyXGetXFreeableMarker.providedBy(self.context)

class ShipRenderer(ship.Renderer):
    """"""
    
    @property
    def available(self):
        """"""
        return self.marker.providedBy(self.context) and \
            not IDiscountableMarker.providedBy(self.context) and \
            not IBuyXGetXFreeableMarker.providedBy(self.context)





from getpaid.discount.browser.interfaces import IDiscountable
from getpaid.discount.browser.interfaces import IDiscountableMarker


from Products.CMFCore.utils import getToolByName

def checkDiscount(self, event):
    """
    Check if an item has a discount on it and 
    change the total depending on that
    """
    item = event.object
    ref_obj = item.resolve()
    
    if IDiscountableMarker.providedBy(ref_obj):
        adapter_obj = IDiscountable(ref_obj)
        discount_value = adapter_obj.getDiscountValue()
        if discount_value != 0.0:
            discount_type = adapter_obj.getDiscountType()
            if discount_type == 'Dollars Off':
                item.cost = item.cost - discount_value
            else:
                item.cost = item.cost - item.cost*discount_value/100



from Products.statusmessages.interfaces import IStatusMessage

from zope.component import getMultiAdapter
from Products.CMFPlone import PloneMessageFactory as _
from zope import component
from zope import interface
from zope.formlib import form
from zope.interface import implements
from zope.annotation.interfaces import IAnnotations

from zope.component import getUtility

from Products.Five.formlib.formbase import EditForm
from Products.Five.utilities import marker
from Products.Five.browser import BrowserView

from getpaid.discount.browser.interfaces import IDiscountable
from getpaid.discount.browser.interfaces import IBuyXGetXFreeable
from getpaid.discount.browser.interfaces import ICodeDiscountable
from getpaid.discount.browser.interfaces import IDiscountableMarker
from getpaid.discount.browser.interfaces import IBuyXGetXFreeableMarker
from getpaid.discount.browser.interfaces import ICodeDiscountableMarker

from Products.PloneGetPaid.interfaces import IPayableMarker

from getpaid.core import interfaces
from getpaid.core.interfaces import IShoppingCartUtility

class DiscountForm(EditForm):
    """
    """
    form_fields = form.FormFields(IDiscountable)
    marker = IDiscountableMarker
    actions = form.Actions()
    interface = IDiscountable
    
    def allowed( self ):
        adapter = component.queryAdapter( self.context, IPayableMarker)
        return not (adapter is None)

class DiscountEdit(DiscountForm):
    """
    """
    @form.action(_(u'Update', default=u'Update'),name=u'update', condition=form.haveInputWidgets)
    def update_discountable( self, action, data):
        self.handle_edit_action.success_handler( self, action, data )
        message = 'Changes saved.'
#        result = translate(msgid, domain='plone',context=self.request)
        self.request.response.redirect( '%s/view?portal_status_message=%s' % (self.context.absolute_url(), message) )

class DiscountCreation(DiscountForm):
    """
    """

    @form.action(_(u'Activate', default=u'Activate'),name=u'activate',condition=form.haveInputWidgets)
    def activate_discountable( self, action, data):
        #we set up the type as IDiscountable
        interface.alsoProvides(self.context, self.marker)

        self.handle_edit_action.success_handler( self, action, data )
#        message = 'Changes saved.'
#        self.request.response.redirect( '%s/view?portal_status_message=%s' % (self.context.absolute_url(), message) )
        IStatusMessage(self.request).addStatusMessage(_("Changes saved."),
                                                      type="info")
        url = getMultiAdapter((self.context, self.request),
                              name='absolute_url')()
        self.request.response.redirect(self.context.absolute_url())

class DiscountDestruction(BrowserView):
    marker = IDiscountableMarker
    
    def __call__(self):
        marker.erase( self.context, self.marker )
        self.request.response.redirect( '%s/view' % self.context.absolute_url() )

class BuyXGetXFreeEdit(DiscountEdit):
    """
    """
    interface = IBuyXGetXFreeable
    form_fields = form.FormFields(IBuyXGetXFreeable)
    marker = IBuyXGetXFreeableMarker

class BuyXGetXFreeCreation(DiscountCreation):
    """
    """
    interface = IBuyXGetXFreeable
    form_fields = form.FormFields(IBuyXGetXFreeable)
    marker = IBuyXGetXFreeableMarker

class BuyXGetXFreeDestruction(DiscountDestruction):
    marker = IBuyXGetXFreeableMarker

class CodeDiscountableEdit(DiscountEdit):
    """
    """
    interface = ICodeDiscountable
    form_fields = form.FormFields(ICodeDiscountable)
    marker = ICodeDiscountableMarker

class CodeDiscountableCreation(DiscountCreation):
    """
    """
    interface = ICodeDiscountable
    form_fields = form.FormFields(ICodeDiscountable)
    marker = ICodeDiscountableMarker

class CodeDiscountableDestruction(DiscountDestruction):
    marker = ICodeDiscountableMarker

class DiscountAdapter(object):
    """
    """
    implements(IDiscountable)
    
    def __init__(self, context):
        self.context = context
        self.annotations = IAnnotations(context)
        discount_title = self.annotations.get('discount_title', None)
        if discount_title is None:
            self.annotations['discount_title'] = ''
        discount_type = self.annotations.get('discount_type', None)
        if discount_type is None:
            self.annotations['discount_type'] = 'Dollars Off'
        discount_value = self.annotations.get('discount_value', None)
        if discount_value is None:
            self.annotations['discount_value'] = 0.0
    
    def getDiscountTitle(self):
        return self.annotations['discount_title']

    def setDiscountTitle(self, data):
        self.annotations['discount_title'] = data

    def getDiscountType(self):
        return self.annotations['discount_type']

    def setDiscountType(self, data):
        self.annotations['discount_type'] = data
    
    def getDiscountValue(self):
        return self.annotations['discount_value']

    def setDiscountValue(self, data):
        self.annotations['discount_value'] = data
    
    discount_title = property(fget=getDiscountTitle, fset=setDiscountTitle)
    discount_type = property(fget=getDiscountType, fset=setDiscountType)
    discount_value = property(fget=getDiscountValue, fset=setDiscountValue)

class BuyXGetXFreeAdapter(object):
    """
    """
    implements(IBuyXGetXFreeable)
    
    def __init__(self, context):
        self.context = context
        self.annotations = IAnnotations(context)
        discount_title = self.annotations.get('discount_title', None)
        if discount_title is None:
            self.annotations['discount_title'] = ''
        number_to_buy = self.annotations.get('number_to_buy', None)
        if number_to_buy is None:
            self.annotations['number_to_buy'] = 0
        number_free = self.annotations.get('number_free', None)
        if number_free is None:
            self.annotations['number_free'] = 0
    
    def getDiscountTitle(self):
        return self.annotations['discount_title']

    def setDiscountTitle(self, data):
        self.annotations['discount_title'] = data

    def getNumberToBuy(self):
        return self.annotations['number_to_buy']

    def setNumberToBuy(self, data):
        self.annotations['number_to_buy'] = data
    
    def getNumberFree(self):
        return self.annotations['number_free']

    def setNumberFree(self, data):
        self.annotations['number_free'] = data
    
    discount_title = property(fget=getDiscountTitle, fset=setDiscountTitle)
    number_to_buy = property(fget=getNumberToBuy, fset=setNumberToBuy)
    number_free = property(fget=getNumberFree, fset=setNumberFree)

class CodeDiscountableAdapter(object):
    """
    """
    implements(ICodeDiscountable)
    
    def _getDiscountTitle(x):
        def func(self):
            return self.annotations['discount_title_%s' % x]
    
        return func

    def _setDiscountTitle(x):
        def func(self, data):
            self.annotations['discount_title_%s' % x] = data
            
        return func

    def _getDiscountCode(x):
        def func(self):
            return self.annotations['discount_code_%s' % x]

        return func

    def _setDiscountCode(x):
        def func(self, data):
            if data:
                data = data.strip()
            self.annotations['discount_code_%s' % x] = data

        return func
    
    def _getDiscountedPrice(x):
        def func(self):
            return self.annotations['discounted_price_%s' % x]

        return func

    def _setDiscountedPrice(x):
        def func(self, data):
            self.annotations['discounted_price_%s' % x] = data

        return func

    def __init__(self, context):
        self.context = context
        self.annotations = IAnnotations(context)

        for f in [1,2,3]:
            title = 'discount_title_%s' % f
            code = 'discount_code_%s' % f
            price = 'discounted_price_%s' % f

            setattr(self, title, self.annotations.get(title, None))
            if getattr(self, title, None) is None:
                self.annotations[title] = ''

            setattr(self, code, self.annotations.get(code, None))
            if getattr(self, code, None) is None:
                self.annotations[code] = ''

            setattr(self, price, self.annotations.get(price, None))
            if getattr(self, price, None) is None:
                self.annotations[price] = 0.0

    discount_title_1 = property(fget=_getDiscountTitle(1), fset=_setDiscountTitle((1)))
    discount_code_1 = property(fget=_getDiscountCode(1), fset=_setDiscountCode(1))
    discounted_price_1 = property(fget=_getDiscountedPrice(1), fset=_setDiscountedPrice(1))

    discount_title_2 = property(fget=_getDiscountTitle(2), fset=_setDiscountTitle(2))
    discount_code_2 = property(fget=_getDiscountCode(2), fset=_setDiscountCode(2))
    discounted_price_2 = property(fget=_getDiscountedPrice(2), fset=_setDiscountedPrice(2))

    discount_title_3 = property(fget=_getDiscountTitle(3), fset=_setDiscountTitle(3))
    discount_code_3 = property(fget=_getDiscountCode(3), fset=_setDiscountCode(3))
    discounted_price_3 = property(fget=_getDiscountedPrice(3), fset=_setDiscountedPrice(3))

class ApplyDiscountCode(BrowserView):
    """
    """
    # create form that:
    #   - accepts a code
    #   - iterates over cart and checks that an item provides ICodeDiscountable
    #   - it also does not provide IDiscountable
    #   - Creates new IDiscountable set appropiatly and drops item price
    def __call__(self):
        code = self.request.form.get('discount.code', None)
        code = code.strip()

        self.cart = getUtility(IShoppingCartUtility).get(self.context) or {}

        if not self.cart:
            order_id = self.request.get("order_id", None)
            if order_id:
                order_manager = component.getUtility(interfaces.IOrderManager)
                self.cart = order_manager.get(order_id).shopping_cart

        if code is not None and len(code) is not 0 and self.cart:
            
            for item in self.cart.values():

                ref_obj = item.resolve()
                payable_quantity = item.quantity

                annotation = IAnnotations(item)

                if ref_obj \
                   and ICodeDiscountableMarker.providedBy(ref_obj) \
                   and not "getpaid.discount.code" in annotation:

                    adapter_obj = ICodeDiscountable(ref_obj)

                    for f in [1,2,3]:
                        discount_title = 'discount_title_%s' % f
                        discount_code = 'discount_code_%s' % f
                        discounted_price = 'discounted_price_%s' % f

                        if code.lower() == getattr(adapter_obj, discount_code, '').lower():

                            discounted_price = getattr(adapter_obj, discounted_price)
                        
                            annotation["getpaid.discount.code"] = getattr(adapter_obj, discount_code)

                            # Here I want to create a new IDiscountableMarker
                            # I also want to drop the price on this payable_line    
                            annotation["getpaid.discount.code.title"] = getattr(adapter_obj, discount_title)
                            discount = item.cost - discounted_price
                            item.cost = discounted_price
                            total_discount = discount * item.quantity
                            annotation["getpaid.discount.code.discount"] = "%.2f" % (total_discount)

        self.request.response.redirect('@@getpaid-cart')

    



from zope.i18n import translate
from zope.i18nmessageid import MessageFactory
_ = MessageFactory('getpaid.discount')
from Products.Five.browser import BrowserView

from getpaid.discount.browser.interfaces import IDiscountableMarker
from getpaid.discount.browser.interfaces import IBuyXGetXFreeableMarker
from getpaid.discount.browser.interfaces import IDiscountable
from getpaid.discount.browser.interfaces import IBuyXGetXFreeable

from Products.PloneGetPaid.interfaces import PayableMarkerMap

class BaseDiscountPortlet(object):
    """
    """
    def isDiscountable(self):
        return IDiscountableMarker.providedBy(self.context)
    
    def isBuyXGetXFreeable(self):
        return IBuyXGetXFreeableMarker.providedBy(self.context)
    
    def getPrice(self, item):
        """
        returns the price of the payable item
        """
        for marker, iface in PayableMarkerMap.items():
            if marker.providedBy(item):
                payable = iface(item)
                return payable.price
        return None
     
    def hasNormalDiscount(self, item):
        """
        returns a nice display of the discount on the item
        """
        result = None
        price = self.getPrice(item)
        if IDiscountableMarker.providedBy(item) and price:
            adapter_obj = IDiscountable(item)
            discount_value = adapter_obj.getDiscountValue()
            if discount_value != 0.0:
                discount_type = adapter_obj.getDiscountType()
                if discount_type == 'Dollars Off':
                    
                    msgid = _(u"details_discount_off", default= u"${discounted_value}( ${discount_value} off )", mapping={ u"discounted_value" : price - discount_value, u"discount_value": discount_value})
                    result = translate(msgid, domain='getpaid.discount',context=self.request)
                    #result = "$%0.2f ($%0.2f off)" % (price - discount_value, discount_value)
                else:
                    msgid = _(u"details_discount_percentage", default= u"${discounted_value_percentage}(${discount_value} \%off)", mapping={ u"discounted_value_percentage" : price - price*discount_value/100, u"discount_value": discount_value})
                    result = translate(msgid, domain='getpaid.discount',context=self.request)
                    
                    #result = "$%0.2f (%0.0f%s off)" % (price - price*discount_value/100, discount_value, '%')
        return result

    def hasBuyXGetXFreeDiscount(self, item):
        """
        returns a nice display of the discount on the item
        """
        result = None
        if IBuyXGetXFreeableMarker.providedBy(item):
            adapter_obj = IBuyXGetXFreeable(item)
            number_to_buy = adapter_obj.getNumberToBuy()
            number_free = adapter_obj.getNumberFree()
            if number_to_buy != 0 and number_free != 0:
                #result = "Order %d get %d additional free" % (number_to_buy, number_free)
                msgid = _(u"orderx_getyfree", default= u"Order ${number_to_buy} get ${number_free} additional free (${number_total})", mapping={ u"number_to_buy" : number_to_buy, u"number_free": number_free, u"number_total": number_free + number_to_buy})
                result = translate(msgid, domain='getpaid.discount',context=self.request)
                
        return result
    

class DiscountContentPortlet(BrowserView, BaseDiscountPortlet):
    """ Plone 2.5 View methods for the ContentPortlet """
    
    def __init__(self, *args, **kw):
        super(BrowserView, self).__init__(*args, **kw)
        
        
class PGPContentPortlet(BrowserView):
    """ Plone 2.5 override of the View methods for the ContentPortlet """
    payable = None
    def __init__( self, *args, **kw):
        super( BrowserView, self).__init__( *args, **kw)

        found = False
        for marker, iface in PayableMarkerMap.items():
            if marker.providedBy( self.context ):
                found = True
                break

        if found:
            self.payable = iface( self.context )
        
    def isPayable(self):
        return self.payable is not None and \
            not IDiscountableMarker.providedBy(self.context) and \
            not IBuyXGetXFreeableMarker.providedBy(self.context)




"""
$Id$

Copyright (c) 2007 - 2008 ifPeople, Kapil Thangavelu, and Contributors
All rights reserved. Refer to LICENSE.txt for details of distribution and use.

Distutils setup
 
"""

import os
from setuptools import setup, find_packages

def read(*rnames):
    return open(os.path.join(os.path.dirname(__file__), *rnames)).read()

version = '0.10'

setup(name='getpaid.discount',
      version=version,
      license = 'ZPL2.1',
      description='package for plone getpaid handling discounts',
      long_description = (
        read('README.txt')
        + '\n' +
        read('CHANGES.txt')
        + '\n' +
        'Detailed Documentation\n'
        '**********************\n'
        + '\n' +
        read('src', 'getpaid', 'discount', 'README.txt')
        + '\n' +
        'Download\n'
        '**********************\n'
        ),
      # Get more strings from http://www.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
        'Framework :: Plone',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        ],
      keywords='',
      author='Six Feet Up, Inc.',
      author_email='info@sixfeetup.com',
      url='http://code.google.com/p/getpaid',
      packages=find_packages('src'),
      package_dir={'':'src'},
      namespace_packages=['getpaid'],
      include_package_data=True,
      zip_safe=False,
      install_requires=['setuptools',
                        'five.formlib',
                        'getpaid.core>=0.7.6dev',
                        'plone.app.portlets',
                        'plone.portlets',
                        'zope.annotation',
                        'zope.component',
                        'zope.i18n',
                        'zope.i18nmessageid',
                        'zope.interface',
                        'zope.formlib',
                        'Products.CMFCore',
                        'Products.PloneGetPaid>=0.8.6dev',
                         ],
      )
